# flake8: noqa

# import all models into this package
# if you have many models here with many references from one model to another this may
# raise a RecursionError
# to avoid this, import only the models that you directly need like:
# from from identitylib.card_client.model.pet import Pet
# or import this package, but before doing it, use:
# import sys
# sys.setrecursionlimit(n)

from identitylib.card_client.model.api_exception import APIException
from identitylib.card_client.model.available_barcode import AvailableBarcode
from identitylib.card_client.model.card import Card
from identitylib.card_client.model.card_identifier import CardIdentifier
from identitylib.card_client.model.card_logo import CardLogo
from identitylib.card_client.model.card_note import CardNote
from identitylib.card_client.model.card_request import CardRequest
from identitylib.card_client.model.card_request_summary import CardRequestSummary
from identitylib.card_client.model.card_summary import CardSummary
from identitylib.card_client.model.inline_object import InlineObject
from identitylib.card_client.model.inline_object1 import InlineObject1
from identitylib.card_client.model.inline_response200 import InlineResponse200
from identitylib.card_client.model.inline_response2001 import InlineResponse2001
from identitylib.card_client.model.inline_response2002 import InlineResponse2002
from identitylib.card_client.model.inline_response2003 import InlineResponse2003
from identitylib.card_client.model.inline_response2004 import InlineResponse2004
from identitylib.card_client.model.inline_response2005 import InlineResponse2005
from identitylib.card_client.model.inline_response2006 import InlineResponse2006
from identitylib.card_client.model.inline_response2007 import InlineResponse2007
from identitylib.card_client.model.inline_response2007_results import InlineResponse2007Results
from identitylib.card_client.model.v1beta1_card_requests_id_identifiers import V1beta1CardRequestsIdIdentifiers
from identitylib.card_client.model.validation_error import ValidationError



from unittest import TestCase, mock
from identitylib.api_client_mixin import ClientCredentialsConfigurationMixin


class TestApiClientConfigurationMixin(TestCase):
    class MockResponse:
        """ mock the api post response """

        def __init__(self, response_status_code, response_text, response_data):
            self.status_code = response_status_code
            self.text = response_text
            self.response_data = response_data

        def __iter__(self):
            return self

        def __next__(self):
            return self

        def json(self):
            return self.response_data

    @mock.patch('identitylib.api_client_mixin.requests.post', side_effect=MockResponse(
        200, "api responds 200", {'access_token': 'new_access_token'}
    ))
    def test_refresh_access_token(self, mock_post):
        """ check the mixin refreshes access token """

        configuration = ClientCredentialsConfigurationMixin(
            'client_key', 'client_secret', 'access_token_url'
        )
        configuration._refresh_access_token()

        self.assertEqual('new_access_token', configuration.access_token)

    @mock.patch('identitylib.api_client_mixin.requests.post', side_effect=MockResponse(
        401, "api responds 401",
        {"fault": {"faultstring": "Invalid access token",
                   "detail": {"errorcode": "oauth.v2.InvalidAccessToken"}}}
    ))
    def test_refresh_access_token_fails(self, mock_post):
        """ check the mixin correctly raises exception when server responds 401 """

        with self.assertRaises(RuntimeError):
            configuration = ClientCredentialsConfigurationMixin(
                'client_key', 'client_secret', 'access_token_url'
            )
            configuration._refresh_access_token()

    @mock.patch('identitylib.api_client_mixin.requests.post', side_effect=MockResponse(
        200, "no access_token", {'no_access_token': 'not_an_access_token'}
    ))
    def test_refresh_access_token_no_token(self, mock_post):
        """ check the mixin correctly raises exception when no access_token in response """

        with self.assertRaises(RuntimeError):
            configuration = ClientCredentialsConfigurationMixin(
                'client_key', 'client_secret', 'access_token_url'
            )
            configuration._refresh_access_token()



import requests
from logging import getLogger

LOG = getLogger(__name__)


class ClientCredentialsConfigurationMixin(object):

    """ mixin to support client_credentials authentication flow """

    def __init__(self, client_key, client_secret, access_token_url):

        self.client_key = client_key
        self.client_secret = client_secret
        self.access_token_url = access_token_url

        self.access_token = None
        self.refresh_api_key_hook = self._refresh_access_token()

    def _refresh_access_token(self):
        """ refresh the access token """

        LOG.debug("Refreshing API access token")

        auth = requests.auth.HTTPBasicAuth(self.client_key, self.client_secret)
        data = {"grant_type": "client_credentials"}

        response = requests.post(self.access_token_url, data=data, auth=auth)

        if response.status_code != 200 or not response.json().get('access_token'):
            raise RuntimeError(
                f'Access token request failed: {response.status_code}, {response.text}'
            )

        self.access_token = response.json()['access_token']



from identitylib.card_client.configuration import Configuration
from identitylib.api_client_mixin import ClientCredentialsConfigurationMixin


class CardClientConfiguration(Configuration, ClientCredentialsConfigurationMixin):

    def __init__(self, client_key, client_secret, access_token_url, *args, **kwargs):

        Configuration.__init__(self, *args, **kwargs)
        ClientCredentialsConfigurationMixin.__init__(
            self, client_key, client_secret, access_token_url
        )
